@@ -1,6 +1,6 @@
 """Luminosity Function Constructor and Modeller
 
-This script allows the user to construct Galaxian Luminosity Functions using the 1/Vmax estimator and Schechter model. 
+This script allows the user to construct and model Galaxian Luminosity Functions using the 1/Vmax estimator and Schechter function. 
 
 Rest-frame magnitudes and spatial variance on the counts can be obtained. 
 Plotting function for easy visualisation are included.
@@ -12,97 +12,104 @@ functions:
     * get_maggy_inv_var - returns inverse variances on maggies
     * get_rest_mag - converts apparent magnitudes into rest-frame magnitudes
     * get_volume - returns comoving volume of input survey area and redshift
-    * get_binned_phi - bins and weighs galaxy counts per magnitude implementing the 1/Vmax estimator
-    * get_patches_centers - finds centers of equally distributed and equally sized patches over survey area
-    * get_patches - divides survey into equally distributed and equally sized patches
-    * get_binned_phi_error - returns spatial variance on galaxy number density per magnitude
-    * plot_LF - plots the 1/Vmax weighted luminosity function from data, binned by magnitude.
-    * analyse_LF_by_colour - plots the 1/Vmax weighted luminosity function from data, binned by magnitude and filtered by galaxy colours
-    * SchechterMagModel - single Schechter luminosity function in terms of magnitude
-    * DoubleSchechterMagModel - double Schechter luminosity function in terms of magnitude 
+    * get_binned_phi - bins and weights galaxy counts per magnitude by 1/Vmax
+    * get_patches_centers - returns centers of equal patches over survey area
+    * get_patches - divides survey into equal patches
+    * get_binned_phi_error - returns spatial variance of the luminosity function 
+    * plot_LF - plots magnitude-binned and 1/Vmax weighted luminosity function
+    * analyse_LF_by_colour - plots luminosity functions filtered by galaxy colour
+    * SchechterMagModel - single Schechter function in terms of magnitude
+    * DoubleSchechterMagModel - double Schechter function in terms of magnitude 
     * get_gof - returns reduced chi squared estimate of goodness of fit
-    * get_schechter_phi - best fits single Schechter model on data 
-    * get_double_schechter_phi - best fits double Schechter model on data 
+    * get_schechter_phi - best fits single Schechter function on data 
+    * get_double_schechter_phi - best fits double Schechter function on data 
 
 """
 # -----------------------
 # Package Imports
 # -----------------------
-from scipy.optimize import curve_fit
+import numpy as np
+
+from typing import Tuple
 
 import matplotlib
 import matplotlib.cm as cm
 import matplotlib.pyplot as plt
 
+from scipy.optimize import curve_fit
+
 import kmeans_radec
 from kmeans_radec import KMeans, kmeans_sample
-import numpy as np
+
 from astropy.cosmology import FlatLambdaCDM
 import astropy.units as u
 cosmo = FlatLambdaCDM(H0=70 * u.km / u.s / u.Mpc, Tcmb0=2.725 * u.K, Om0=0.3)
 
-from typing import Tuple
-
 # -----------------------
 # Methods
 # -----------------------
 def get_maggy(app_mag_list: np.ndarray) -> np.ndarray:
     """
     Converts magnitudes into maggies.
-
+    
     Parameters
     ----------
-    app_mag_list : np.ndarray, list
-        numpy array of all apparent magnitude values
+    app_mag_list : np.ndarray
+        apparent magnitude of each data point (galaxy)
 
     Returns
     -------
-    np.ndarray, list
-        numpy array of all corresponding maggy values
+    np.ndarray
+        all corresponding maggy values
 
     """
     maggies_list = 10**(app_mag_list / (-2.5))
     return maggies_list
 
-def get_maggy_inv_var(maggies_list: np.ndarray, app_mag_err_list: np.ndarray) -> np.ndarray:
+def get_maggy_inv_var(maggies_list: np.ndarray, 
+                      app_mag_err_list: np.ndarray) -> np.ndarray:
     """
     Returns inverse variances on maggies using maggies and magnitude errors.
-
+    
     Parameters
     ----------
-    maggies_list : np.ndarray, list
-        numpy array of all maggy values
-    app_mag_err_list : np.ndarray, list
-        numpy array of all corresponding errors on apparent magnitude values
+    maggies_list : np.ndarray
+        maggy value of each data point (galaxy)
+    app_mag_err_list : np.ndarray
+        all corresponding errors on apparent magnitude values
 
     Returns
     -------
-    np.ndarray, list
-        numpy array of all correspoding maggy inverse variance values 
+    np.ndarray
+        all correspoding maggy inverse variance values 
 
     """
+    
     inv_var_list = (0.4 * np.log(10) * maggies_list * app_mag_err_list)**(-2)
     return inv_var_list
 
-def get_rest_mag(redshift_list: np.ndarray, app_mag_list: np.ndarray, maggy_ratio_list: np.ndarray) -> np.ndarray:
+def get_rest_mag(redshift_list: np.ndarray, 
+                 app_mag_list: np.ndarray, 
+                 maggy_ratio_list: np.ndarray) -> np.ndarray:
     """
-    Converts apparent magnitudes into rest-frame magnitudes using apparent magnitudes, redshifts and maggy ratios.
-
+    Converts apparent magnitudes into rest-frame magnitudes using the apparent magnitudes, redshifts and maggy ratios.
+    
     Parameters
     ----------
-    redshift_list : np.ndarray, list
-        numpy array of all redshifts
-    app_mag_list : np.ndarray, list
-        numpy array of all corresponding apparent magnitudes
-    maggy_ratio_list : np.ndarray, list
-        numpy array of all corresponding maggy ratios
+    redshift_list : np.ndarray
+        redshift of each data point (galaxy)
+    app_mag_list : np.ndarray
+        all corresponding apparent magnitudes
+    maggy_ratio_list : np.ndarray
+        all corresponding maggy ratios
 
     Returns
     -------
-    np.ndarray, list
-        numpy array of all corresponding rest-frame magnitudes
+    np.ndarray
+        all corresponding rest-frame magnitudes
 
     """
+    
     # calculate luminosity distance
     lum_dist_list = cosmo.luminosity_distance(redshift_list).value
     print('\tLuminosity distance calculated.')
@@ -121,23 +128,25 @@ def get_rest_mag(redshift_list: np.ndarray, app_mag_list: np.ndarray, maggy_rati
 
     return rest_mag_list
 
-def get_volume(survey_area: float, redshift_list: np.ndarray) -> np.ndarray:
+def get_volume(survey_area: float, 
+               redshift_list: np.ndarray) -> np.ndarray:
     """
-    Returns comoving volume of input survey area and redshift
-
+    Returns comoving volume of input survey area and redshift.
+    
     Parameters
     ----------
     survey_area : float
-        float value of survey area in sq. deg. 
-    redshift_list : np.ndarray, list
-        numpy array of all redshifts
+        survey area in sq. deg. 
+    redshift_list : np.ndarray
+        redshift of each data point (galaxy)
 
     Returns
     -------
-    np.ndarray, list
-        numpy array of all corresponding comoving volumes
+    np.ndarray
+        all corresponding comoving volumes
 
     """
+    
     # calculate comoving distance
     com_dist_list = cosmo.comoving_distance(redshift_list).value
     print('\tComoving distance calculated.')
@@ -152,28 +161,29 @@ def get_volume(survey_area: float, redshift_list: np.ndarray) -> np.ndarray:
 
     return vol_list
 
-def get_binned_phi(rest_mag_list: np.ndarray, Vmax_list: np.ndarray, n_mag_bins: int) -> np.ndarray:
+def get_binned_phi(rest_mag_list: np.ndarray, 
+                   Vmax_list: np.ndarray, 
+                   n_mag_bins: int) -> np.ndarray:
     """
-    Bins and weighs galaxy counts per magnitude implementing the 1/Vmax estimator.
-    Returns phi using rest-frame magnitude, maximum observed volume and number of bins.
-
+    Bins and weighs galaxy counts per magnitude implementing the 1/Vmax estimator. Returns phi using rest-frame magnitude, maximum observed volume and the number of bins.
+    
     Parameters
     ----------
     rest_mag_list : np.ndarray
-        numpy array of all rest-frame magnitudes
+        rest-frame magnitude of each data point (galaxy)
     Vmax_list : np.ndarray
-        numpy array of all corresponding maximum volumes
+        all corresponding maximum volumes
     n_mag_bins: int
-        integer value of number of magnitude bins required
-
+        number of magnitude bins required
+        
     Returns
     -------
     np.ndarray
-        numpy array of mid-magnitude (i.e. x) value of each bin
+        mid-magnitude (i.e. x) value of each bin
     np.ndarray
-        numpy array of magnitude-width/2 (i.e. x-error) value of each bin
+        magnitude-width/2 (i.e. x-error) value of each bin
     np.ndarray
-        numpy array of phi (i.e. y) value of each bin (with h = 0.7)
+        phi (i.e. y) value of each bin (with h = 0.7)
 
     """
 
@@ -230,24 +240,35 @@ def get_binned_phi(rest_mag_list: np.ndarray, Vmax_list: np.ndarray, n_mag_bins:
 
     return mid_M_list, M_err_list, phi_list
 
-def get_patches_centers(uniform_random_RA_list: np.ndarray,
-                        uniform_random_DEC_list: np.ndarray,
-                        n_patches: int,
-                        survey='kids'):
+def get_patches_centers(uniform_random_RA_list: np.ndarray, 
+                        uniform_random_DEC_list: np.ndarray, 
+                        n_patches: int, 
+                        survey='kids',
+                        max_iterations=100, 
+                        tolerance=1.0e-5) -> np.ndarray:
     """
-    Divides a uniform random survey into equally distributed and equally sized patches.
-    Returns centers as [RA,Dec] of n_patches from RA, Dec and number of patches.
-
+    Divides the input uniform random survey into equally distributed and equally sized patches. Returns centers as [RA,Dec] of n_patches from RA, Dec and number of patches.
+    
     Parameters
     ----------
-    (1) numpy array of all RA values in a uniform random catalogue
-    (2) numpy array of all corresponding Dec values in a uniform random catalogue
-    (3) integer value of number of patches required
-    (4) string with survey name - only change if survey area covers/connects over 320 degree RA and does not connect over 360 to 0 degree RA
+    uniform_random_RA_list : np.ndarray
+        RA values of each data point (galaxy) in a uniform random catalogue
+    uniform_random_DEC_list : np.ndarray
+        all corresponding Dec values in the uniform random catalogue
+    n_patches : int
+        number of equal survey area patches required
+    survey : str, optional
+        survey name - only change if survey area covers/connects over 320 degree RA and does not connect over 360 to 0 degree RA 
+    max_iterations: int, optional
+        maximum number of iterations to run
+    tolerance: float, optional
+        relative change in the average distance to centers, signifies convergence
 
     Returns
-    ------- 
-    (1) (n_patches, 2) numpy array tuple of patch center guesses [RA,Dec]
+    -------
+    np.ndarray
+        (n_patches, 2) array of patch center guesses [RA,Dec]
+        
     """
 
     # MAKE SURE ALL PATCHES ARE SITCHED ON SKY
@@ -268,46 +289,45 @@ def get_patches_centers(uniform_random_RA_list: np.ndarray,
     # DIVIDE uniform_random_X INTO EQUAL n_patches
     uniform_random_km = kmeans_sample(uniform_random_X,
                                       n_patches,
-                                      maxiter=100,
-                                      tol=1.0e-5)
+                                      max_iterations,
+                                      tolerance)
     center_guesses = uniform_random_km.centers
     ra_guesses = center_guesses[:, 0]
     dec_guesses = center_guesses[:, 1]
-    centers_tuple = np.column_stack((ra_guesses, dec_guesses))
-    return centers_tuple
+    centers_array = np.column_stack((ra_guesses, dec_guesses))
+    return centers_array
 
-def get_patches(RA_list: np.ndarray,
-                DEC_list: np.ndarray,
-                n_patches: int,
-                center_guesses: np.ndarray,
-                survey='kids',
-                numba_installed=True,
+def get_patches(RA_list: np.ndarray, 
+                DEC_list: np.ndarray, 
+                n_patches: int, 
+                center_guesses: np.ndarray, 
+                survey='kids', 
+                numba_installed=True, 
                 plot_savename='none') -> np.ndarray:
     """
-    Divides survey into equally distributed and equally sized patches.
-    Returns labels for patches from RA, Dec, Number of patches and patch center guesses.
-
+    Divides survey into equally distributed and equally sized patches. Returns labels for patches from RA, Dec, number of patches and patch center guesses.
+    
     Parameters
     ----------
     RA_list : np.ndarray
-        numpy array of all RA values
+        RA values of each data point (galaxy)
     DEC_list : np.ndarray
-        numpy array of all corresponding Dec values
-    n_patches: int
-        integer value of number of patches required
-    center_guesses: np.ndarray
-        (n_patches, 2) numpy array of patch center guesses [RA,Dec]
-    survey: str
-        string with survey name
-    numba_installed: bool
-        boolean - mark as False if numba is not installed
-    plot_savename: str
-        string with name and extension to save plot as
-
+        all corresponding Dec values
+    n_patches : int
+        number of equal survey area patches required
+    center_guesses : np.ndarray
+        (n_patches, 2) array of patch center guesses [RA, Dec]
+    survey : str, optional
+        survey name - only change if survey area covers/connects over 320 degree RA and does not connect over 360 to 0 degree RA 
+    numba_installed : bool, optional
+        mark as False if numba is not installed
+    plot_savename : str, optional
+        name and extension to save plot as
+        
     Returns
     -------
     np.ndarray
-        numpy array of patch assignment labels for each RA entry
+        array of patch assignment label for each data point
 
     """
 
@@ -356,8 +376,8 @@ def get_patches(RA_list: np.ndarray,
                         color=colors[n],
                         s=1)
         # if 'gama' in datasetname:
-        	# plt.xlim(120, 240)
-       		# plt.ylim(-10, 10)
+            # plt.xlim(120, 240)
+            # plt.ylim(-10, 10)
         # if 'kids' in datasetname:
         	# plt.xlim(-50, 250)
         	# plt.ylim(-40, 10)
@@ -370,31 +390,31 @@ def get_patches(RA_list: np.ndarray,
     return labels
 
 def get_binned_phi_error(rest_mag_list: np.ndarray, 
-			 Vmax_list: np.ndarray, 
-		         labels: np.ndarray, 
-			 n_patches: int, 
-			 n_mag_bins: int) -> np.ndarray:
+                         Vmax_list: np.ndarray, 
+                         labels: np.ndarray, 
+                         n_patches: int, 
+                         n_mag_bins: int) -> np.ndarray:
     """
-    Spatial variance on galaxy number density per magnitude.
-    Returns error on phi from rest-frame magnitude, maximum observed volume, 
-    labels, number of patches and number of bins.
-
+    Spatial variance on galaxy number density per magnitude. Returns error on phi from rest-frame magnitude, maximum observed volume, labels, number of patches and number of bins.
+    
     Parameters
     ----------
-    rest_mag_list: np.ndarray
-	numpy array of all rest-frame magnitudes
-    Vmax_list: np.ndarray
-	numpy array of all corresponding maximum volumes
-    labels: np.ndarray
-	numpy array of all corresponding patch assignment labels
-    n_patches: int
-	integer value of number of patches required
-    n_mag_bins: int
-	integer value of number of magnitude bins required
+    rest_mag_list : np.ndarray
+        rest-frame magnitude of each data point (galaxy)
+    Vmax_list : np.ndarray
+        all corresponding maximum volumes
+    labels : np.ndarray
+        all corresponding survey patch assignment labels
+    n_patches : int
+        number of equal survey area patches required
+    n_mag_bins : int
+        number of magnitude bins required
+    
     Returns
     -------
     np.ndarray
-	numpy array of phi error (i.e. y-error) value of each bin
+        phi error (i.e. y-error) value of each bin
+        
     """
 
     # GET PHI VALUES USING ONLY VALUES IN EACH PATCH
@@ -412,52 +432,53 @@ def get_binned_phi_error(rest_mag_list: np.ndarray,
 
     return phi_err_list
 
-def plot_LF(rest_mag_list: np.ndarray,
-            Vmax_list: np.ndarray,
-            n_mag_bins: int,
-            RA_list: np.ndarray,
-            DEC_list: np.ndarray,
-            n_patches: int,
-            center_guesses: np.ndarray,
-            survey='kids',
-            numba_installed=True,
+def plot_LF(rest_mag_list: np.ndarray, 
+            Vmax_list: np.ndarray, 
+            n_mag_bins: int, 
+            RA_list: np.ndarray, 
+            DEC_list: np.ndarray, 
+            n_patches: int, 
+            center_guesses: np.ndarray, 
+            survey='kids', 
+            numba_installed=True, 
             plot_savename='none') -> np.ndarray:
     """
     Plots the 1/Vmax weighted luminosity function from data, binned by magnitude.
-
+    
     Parameters
     ----------
-    rest_mag_list: np.ndarray
-	numpy array of all rest-frame magnitudes
-    Vmax_list: np.ndarray
-	numpy array of all corresponding maximum volumes
-    n_mag_bins: int
-	integer value of number of magnitude bins required
-    RA_list: np.ndarray
-	numpy array of all RA values
-    DEC_list: np.ndarray
-	numpy array of all corresponding Dec values
-    n_patches: int
-	integer value of number of patches required
-    center_guesses: np.ndarray
-	(n_patches, 2) numpy array of patch center guesses [RA,Dec]
-    survey: str
-	string with survey name. Only change if survey area covers/connects over 320 degree RA and does not connect over 360 to 0 degree RA. 
-    numba_installed: bool
-        boolean - mark as False if numba is not installed
-    plot_savename: str
-	string with name and extension to save plot as
+    rest_mag_list : np.ndarray
+        rest-frame magnitude of each data point (galaxy)
+    Vmax_list : np.ndarray
+        all corresponding maximum volumes
+    n_mag_bins : int
+        number of magnitude bins required
+    RA_list : np.ndarray
+        all corresponding RA values
+    DEC_list : np.ndarray
+        all corresponding Dec values
+    n_patches : int
+        number of equal survey area patches required
+    center_guesses : np.ndarray
+        (n_patches, 2) equal survey area patch center guesses [RA,Dec]
+    survey : str, optional
+        survey name - only change if survey area covers/connects over 320 degree RA and does not connect over 360 to 0 degree RA 
+    numba_installed : bool, optional
+        mark as False if numba is not installed
+    plot_savename : str, optional
+        name and extension to save plot as
     
     Returns
     -------
     np.ndarray
-	numpy array of mid-magnitude (i.e. x) value of each bin
+        mid-magnitude (i.e. x) value of each bin
     np.ndarray
-	numpy array of magnitude-width/2 (i.e. x-error) value of each bin
+        magnitude-width/2 (i.e. x-error) value of each bin
     np.ndarray
-	numpy array of phi (i.e. y) value of each bin (with h = 0.7)
+        phi (i.e. y) value of each bin (with h = 0.7)
     np.ndarray
-	numpy array of phi error (i.e. y-error) value of each bin
+        phi error (i.e. y-error) value of each bin
+        
     """
 
     # phi
@@ -500,74 +521,75 @@ def plot_LF(rest_mag_list: np.ndarray,
 
     return M_list, M_err_list, phi_list, phi_err_list
 
-def analyse_LF_by_colour(dichotomy_slope: float,
-                         dichotomy_intercept: float,
-                         rest_mag_list: np.ndarray,
-                         Vmax_list: np.ndarray,
-                         n_mag_bins: int,
-                         RA_list: np.ndarray,
-                         DEC_list: np.ndarray,
-                         n_patches: int,
-                         center_guesses: np.ndarray,
-                         survey='kids',
-                         numba_installed=True,
+def analyse_LF_by_colour(dichotomy_slope: float, 
+                         dichotomy_intercept: float, 
+                         rest_mag_list: np.ndarray, 
+                         Vmax_list: np.ndarray, 
+                         n_mag_bins: int, 
+                         RA_list: np.ndarray, 
+                         DEC_list: np.ndarray, 
+                         n_patches: int, 
+                         center_guesses: np.ndarray, 
+                         survey='kids', 
+                         numba_installed=True, 
                          plot_savename='none') -> np.ndarray:
     """
-    Plots the 1/Vmax weighted luminosity function from data, binned by magnitude and filtered by galaxy colours.
-
+    Plots the 1/Vmax weighted luminosity function from data, binned by magnitude and filtered by galaxy colours. The galaxy colours are filtered by red and blue with the help of the input colour dichotomy line parameters. The colour dichotomy line parameters can be inferred from a CMD plot
+    
     Parameters
     ----------
-    dichotomy_slope: float
-	float value of the slope of the colour dichotomy line
-    dichotomy_intercept: float
-	float value of the intercept of the colour dichotomy line
-    rest_mag_list: np.ndarray
-	numpy array of all rest-frame magnitudes
-    Vmax_list: np.ndarray
-	numpy array of all corresponding maximum volumes
-    n_mag_bins: int
-	integer value of number of magnitude bins required
-    RA_list: np.ndarray
-	numpy array of all RA values
-    DEC_list: np.ndarray
-	numpy array of all corresponding Dec values
-    n_patches: int
-	integer value of number of patches required
-    center_guesses: np.ndarray
-	(n_patches, 2) numpy array of patch center guesses [RA,Dec]
-    survey: str
-	string with survey name. Only change if survey area covers/connects over 320 degree RA and does not connect over 360 to 0 degree RA
-    numba_installed=True: bool
-	boolean - mark as False if numba is not installed
-    plot_savename: str
-	string with name and extension to save plot as
+    dichotomy_slope : float
+        slope of the colour dichotomy line
+    dichotomy_intercept : float
+        intercept of the colour dichotomy line
+    rest_mag_list : np.ndarray
+        rest-frame magnitude of each data point (galaxy)
+    Vmax_list : np.ndarray
+        all corresponding maximum volumes
+    n_mag_bins : int
+        number of magnitude bins required
+    RA_list : np.ndarray
+        all coressponding RA values
+    DEC_list : np.ndarray
+        all corresponding Dec values
+    n_patches : int
+        number of patches required
+    center_guesses : np.ndarray
+        (n_patches, 2) equal survey area patch center guesses [RA,Dec]
+    survey : str, optional
+        survey name - only change if survey area covers/connects over 320 degree RA and does not connect over 360 to 0 degree RA 
+    numba_installed : bool, optional
+        mark as False if numba is not installed
+    plot_savename : str, optional
+        name and extension to save plot as
 
     Returns
     -------
     np.ndarray
-	numpy array of mid-magnitude (i.e. x) value of each bin for LF of all galaxies
+        all galaxies' LF's mid-magnitude (i.e. x) value of each bin
     np.ndarray
-	numpy array of magnitude-width/2 (i.e. x-error) value of each bin for LF of all galaxies
+        all galaxies' LF's magnitude-width/2 (i.e. x-error) value of each bin
     np.ndarray
-	numpy array of phi (i.e. y) value of each bin (with h = 0.7) for LF of all galaxies
+        all galaxies' LF's phi (i.e. y) value of each bin (with h = 0.7)
     np.ndarray
-	numpy array of phi error (i.e. y-error) value of each bin for LF of all galaxies
+        all galaxies' LF's phi error (i.e. y-error) value of each bin
     np.ndarray
-	numpy array of mid-magnitude (i.e. x) value of each bin for LF of red galaxies
+        red galaxies' LF's mid-magnitude (i.e. x) value of each bin
     np.ndarray
-	numpy array of magnitude-width/2 (i.e. x-error) value of each bin for LF of red galaxies
+        red galaxies' LF's magnitude-width/2 (i.e. x-error) value of each bin
     np.ndarray
-	numpy array of phi (i.e. y) value of each bin (with h = 0.7) for LF of red galaxies
+        red galaxies' LF's phi (i.e. y) value of each bin (with h = 0.7)
     np.ndarray
-	numpy array of phi error (i.e. y-error) value of each bin for LF of red galaxies
+        red galaxies' LF's phi error (i.e. y-error) value of each bin
     np.ndarray
-	numpy array of mid-magnitude (i.e. x) value of each bin for LF of blue galaxies
+        blue galaxies' LF's mid-magnitude (i.e. x) value of each bin
     np.ndarray
-	numpy array of magnitude-width/2 (i.e. x-error) value of each bin for LF of blue galaxies
+        blue galaxies' LF's magnitude-width/2 (i.e. x-error) value of each bin
     np.ndarray
-	numpy array of phi (i.e. y) value of each bin (with h = 0.7) for LF of blue galaxies
+        blue galaxies' LF's phi (i.e. y) value of each bin (with h = 0.7)
     np.ndarray
-	numpy array of phi error (i.e. y-error) value of each bin for LF of blue galaxies
+        blue galaxies' LF's phi error (i.e. y-error) value of each bin
+        
     """
 
     dichotomy_line = dichotomy_slope * rest_mag_list + dichotomy_intercept
@@ -639,25 +661,29 @@ def analyse_LF_by_colour(dichotomy_slope: float,
 
     return M_list, M_err_list, phi_list, phi_err_list, red_M_list, red_M_err_list, red_phi_list, red_phi_err_list, blue_M_list, blue_M_err_list, blue_phi_list, blue_phi_err_list
 
-def SchechterMagModel(M_list: np.ndarray, M_star: float, phi_star: float, alpha: float) -> np.ndarray:
+def SchechterMagModel(M_list: np.ndarray, 
+                      M_star: float, 
+                      phi_star: float, 
+                      alpha: float) -> np.ndarray:
     """
-    Single Schechter luminosity function in terms of magnitude from 3 free parameters of the model.    
-
+    Single Schechter luminosity function in terms of magnitude from 3 free parameters of the model.
+    
     Parameters
     ----------
-    M_list: np.ndarray
-	numpy array of magnitudes (i.e. x)
-    M_star: float
-	float value of parameter M_star
-    phi_star: float 
-	float value of parameter phi_star
-    alpha: float
-	float value of parameter alpha
+    M_list : np.ndarray
+        array of magnitudes (i.e. x)
+    M_star : float
+        model parameter M_star
+    phi_star : float 
+        model parameter phi_star
+    alpha : float
+        model parameter alpha
 
     Returns
     -------
     np.ndarray
-	numpy array of Schechter modelled phi (i.e. y)
+        array of Schechter modelled phi (i.e. y)
+        
     """
 
     # FACTOR
@@ -681,29 +707,35 @@ def SchechterMagModel(M_list: np.ndarray, M_star: float, phi_star: float, alpha:
 
     return phi_list
 
-def DoubleSchechterMagModel(M_list: np.ndarray, M_star: float, phi_star1: float, alpha1: float, phi_star2: float, alpha2: float) -> np.ndarray:
+def DoubleSchechterMagModel(M_list: np.ndarray, 
+                            M_star: float, 
+                            phi_star1: float, 
+                            alpha1: float, 
+                            phi_star2: float, 
+                            alpha2: float) -> np.ndarray:
     """
-    Double Schechter luminosity function in terms of magnitude from 5 free parameters of the model.   
-
+    Double Schechter luminosity function in terms of magnitude from 5 free parameters of the model.
+    
     Parameters
     ----------
-    M_list: np.ndarray
-	numpy array of magnitudes (i.e. x)
-    M_star: float
-	float value of parameter M_star
-    phi_star1: float 
-	float value of parameter phi_star1
-    alpha1: float
-	float value of parameter alpha1
-    phi_star2: float 
-	float value of parameter phi_star2
-    alpha2: float
-	float value of parameter alpha2
+    M_list : np.ndarray
+        array of magnitudes (i.e. x)
+    M_star : float
+        model parameter M_star
+    phi_star1 : float 
+        model parameter phi_star1
+    alpha1 : float
+        model parameter alpha1
+    phi_star2 : float 
+        model parameter phi_star2
+    alpha2 : float
+        model parameter alpha2
 
     Returns
     -------
     np.ndarray
-	numpy array of Double Schechter modelled phi (i.e. y)
+        array of Double Schechter modelled phi (i.e. y)
+        
     """
 
     # FACTOR
@@ -732,24 +764,29 @@ def DoubleSchechterMagModel(M_list: np.ndarray, M_star: float, phi_star1: float,
 
     return phi_list
 
-def get_gof(obs: np.ndarray, err: np.ndarray, exp: np.ndarray, m: int) -> float:
+def get_gof(obs: np.ndarray, 
+            err: np.ndarray, 
+            exp: np.ndarray, 
+            m: int) -> float:
     """
     Returns reduced chi squared estimate of goodness of fit.
-
+    
     Parameters
     ----------
-    obs: np.ndarray
-	numpy array of observed values (phi from the 1/Vmax estimator)
-    err: np.ndarray
-	numpy array of errors on observed values
-    exp: np.ndarray
-	numpy array of expected values (phi from the Schechter function)
-    m: int
-	integer value of number of parameters used to calculate the expected values
+    obs : np.ndarray
+        observed values (e.g. phi from survey data)
+    err : np.ndarray
+        errors on observed values
+    exp : np.ndarray
+        expected values (e.g. phi from the Schechter function)
+    m : int
+        number of parameters used to calculate the expected values
+    
     Returns
     -------
     np.ndarray
-	float value of reduced chi square
+        reduced chi square
+        
     """
 
     residuals = obs - exp
@@ -763,48 +800,50 @@ def get_gof(obs: np.ndarray, err: np.ndarray, exp: np.ndarray, m: int) -> float:
 
     return red_chi_sq
 
-def get_schechter_phi(M_list: np.ndarray,
-                      M_err_list: np.ndarray,
-                      phi_list: np.ndarray,
-                      phi_err_list: np.ndarray,
-                      guesses: np.ndarray,
+def get_schechter_phi(M_list: np.ndarray, 
+                      M_err_list: np.ndarray, 
+                      phi_list: np.ndarray, 
+                      phi_err_list: np.ndarray, 
+                      guesses: np.ndarray, 
                       plot_savename='none') -> Tuple[np.ndarray, float, float, float, float, float, float, float]:
     """
-    Best fits single Schechter model on data. Returns best fit phi and reduced chi squared estimate alongside Schechter parameters and their errors.
-
+    Best fits single Schechter function model on data.
+    Returns best fit phi and reduced chi squared estimate alongside the 3 Schechter parameters and their errors.
+    
     Parameters
     ----------
-    M_list: np.ndarray
-	numpy array of mid magnitude (i.e. x) value of each bin
-    M_err_list: np.ndarray
-	numpy array of magnitudes error (i.e. x-error) value of each bin
-    phi_list: np.ndarray
-	numpy array of phi (i.e. y) value of each bin
-    phi_err_list: np.ndarray
-	numpy array of phi error (i.e. y-error) value of each bin
-    guesses: np.ndarray
-	numpy array of Schechter parameter guesses in order [M_star, phi_star, aplha]
-    plot_savename: str
-	string with name and extension to save plot as
+    M_list : np.ndarray
+        mid magnitude (i.e. x) value of each bin
+    M_err_list : np.ndarray
+        magnitudes error (i.e. x-error) value of each bin
+    phi_list : np.ndarray
+        phi (i.e. y) value of each bin
+    phi_err_list : np.ndarray
+        phi error (i.e. y-error) value of each bin
+    guesses : np.ndarray
+        array of Schechter parameter guesses in order [M_star, phi_star, aplha]
+    plot_savename : str, optional
+        name and extension to save plot as
 
     Returns
     -------
     np.ndarray
-	numpy array of Schechter modelled phi (i.e. y)
+        Schechter modelled phi (i.e. y) of each bin
     float
-	float value of reduced chi square of the fit
+        reduced chi square of the fit
     float
-	float value of fit parameter M_star
+        fit parameter M_star
     float
-	float value of error on fit parameter M_star
+        error on fit parameter M_star
     float
-	float value of fit parameter phi_star
+        fit parameter phi_star
     float
-	float value of error on fit parameter phi_star
+        error on fit parameter phi_star
     float
-	float value of fit parameter alpha
+        fit parameter alpha
     float
-	float value of error on fit parameter alpha
+        error on fit parameter alpha
+        
     """
 
     popt, pcov = curve_fit(SchechterMagModel,
@@ -877,54 +916,57 @@ def get_schechter_phi(M_list: np.ndarray,
 
     return model_phi_list, red_chi_sq, M_star, M_star_err, phi_star, phi_star_err, alpha, alpha_err
 
-def get_double_schechter_phi(M_list: np.ndarray,
-                             M_err_list: np.ndarray,
-                             phi_list: np.ndarray,
-                             phi_err_list: np.ndarray,
-                             guesses: np.ndarray,
+def get_double_schechter_phi(M_list: np.ndarray, 
+                             M_err_list: np.ndarray, 
+                             phi_list: np.ndarray, 
+                             phi_err_list: np.ndarray, 
+                             guesses: np.ndarray, 
                              plot_savename='none') -> Tuple[np.ndarray, float, float, float, float, float, float, float, float, float, float, float]:
     """
+    Best fits double Schechter function model on data. Returns best fit phi and reduced chi squared estimate alongside the 5 Schechter parameters and their errors.
+    
     Parameters
     ----------
-    M_list: np.ndarray
-	numpy array of mid magnitude (i.e. x) value of each bin
-    M_err_list: np.ndarray
-	numpy array of magnitudes error (i.e. x-error) value of each bin
-    phi_list: np.ndarray
-	numpy array of phi (i.e. y) value of each bin
-    phi_err_list: np.ndarray
-	numpy array of phi error (i.e. y-error) value of each bin
-    guesses: np.ndarray
-	numpy array of Schechter parameter guesses in order [M_star, phi_star, aplha]
-    plot_savename: str
-	string with name and extension to save plot as
+    M_list : np.ndarray
+        mid magnitude (i.e. x) value of each bin
+    M_err_list : np.ndarray
+        magnitudes error (i.e. x-error) value of each bin
+    phi_list : np.ndarray
+        phi (i.e. y) value of each bin
+    phi_err_list : np.ndarray
+        phi error (i.e. y-error) value of each bin
+    guesses : np.ndarray
+        array of Schechter parameter guesses in order [M_star, phi_star, aplha]
+    plot_savename : str, optional
+        name and extension to save plot as
 
     Returns
     -------
     np.ndarray
-	numpy array of Schechter modelled phi (i.e. y)
+        Schechter modelled phi (i.e. y) of each bin
     float
-	float value of reduced chi square of the fit
+        reduced chi square of the fit
     float
-	float value of fit parameter M_star 
+        fit parameter M_star 
     float
-	float value of error on fit parameter M_star
+        error on fit parameter M_star
     float
-	float value of fit parameter phi_star_1
+        fit parameter phi_star_1
     float
-	float value of error on fit parameter phi_star_1
+        error on fit parameter phi_star_1
     float
-	float value of fit parameter alpha_1
+        fit parameter alpha_1
     float
-	float value of error on fit parameter alpha_1
+        error on fit parameter alpha_1
     float
-	float value of fit parameter phi_star_2
+        fit parameter phi_star_2
     float
-	float value of error on fit parameter phi_star_2
+        error on fit parameter phi_star_2
     float
-	float value of fit parameter alpha_2
+        fit parameter alpha_2
     float
-	float value of error on fit parameter alpha_2
+        error on fit parameter alpha_2
+        
     """
 
     popt, pcov = curve_fit(DoubleSchechterMagModel,
